import json
import datetime
import cherrypy
import pandas as pd
from threading import Thread

from mapnamindsdk import WS as WS
from mapnamindsdk.Rest import Rest as Rest
import mapnamindsdk.Constants as Constants
from mapnamindsdk.Mapper import Mapper as mapper

class Mind(object):

    @staticmethod
    def removeSignal(signalName, userId):

        try:
            body = {"signalName": signalName,
                    "userId": userId
                    }

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/sdk/remove', params=body)
            jsonResult = post_req.post()

            if jsonResult.get('messageCode') == '0000':
                return True
            else:
                return False

            return jsonResult

        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None


    @staticmethod
    def createSignal(signalName, signalDescription,unitMeasurment,dataType, userId):

        try:
            body = {"signalName": signalName,
                    "description": signalDescription,
                    "unitMeasurment": unitMeasurment,
                    "dataType": dataType,
                    "userId": userId
                    }

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/sdk/create', params=body)
            jsonResult = post_req.post();

            if jsonResult['messageCode']=='0000':
                return True
            else:
                return False


        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None

        # TODO Check Signal duplication with performance signals

    @staticmethod
    def setValue(signalName, value, dateAndTime, userId):

        if dateAndTime.lower() != 'now'.lower():
            Mind._validate(dateAndTime)

        # f = mapper.Mapper.getInstance()

        # signalId = f.SignalMapper[signalName]
        # Request Body
        body = {"signalName": signalName,
                "dateAndTime": dateAndTime,
                "value": value,
                "userId": userId

                }

        # post_req = Rest(f'http://{Constants.SDK_SERVER_IP}:{Constants.SDK_PORT}',
        #                 path='sdk-remote/offline/set', params=body)
        post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                                        path='/sdk/offline/set', params=body)
        jsonResult = post_req.post()

        if jsonResult['messageCode'] == '0000':
            return True
        else:
            return False

        return jsonResult

    # @staticmethod
    # def getValue(signalNames):
    #     '''
    #     Get value from ONLINE table
    #     :return:
    #     '''
    #     try:
    #
    #         f = mapper.Mapper.getInstance()
    #
    #         # Get signalId for given signalName from the Mapper
    #         signalID = int(f.SignalMapper[signalNames])
    #
    #         # Request Body
    #         body = {'ids': [signalID], 'type': "TIMESERIES"}
    #
    #         post_req = Rest(f'http://{Constants.DATASERVICE_SERVER_IP}:{Constants.DATASERVICE_PORT}',
    #                         path='/online/get', params=body)
    #         listResult = post_req.post()
    #
    #
    #         # Convert Json to List/Dict composition
    #         dictResult = json.loads(jsonResult)[0]
    #
    #         return dictResult
    #     except KeyError as err:
    #         print("ERROR: Signal Name {} not found!\n".format(err))
    #     return None

    @staticmethod
    def _mapHistorian2DataFrame(x, signalNames):
        '''
        Map function for converting each row of the given list (x) to dictionary
        :param x: A single row of list
        :param signalNames: list of signal_names for columns title
        :return: Input list rows in dict format
        '''

        try:
        # Create an empty row
            dictCurrentRow = {}

        # add first column of DataFrame table and its value
            dictCurrentRow.update({'time': x['time']})

        # Create rest of columns using list of signal_names and their values
            valueColumns = {signalNames[i]: x['values'][i] for i in range(0, len(signalNames))}

        # Add rest of the columns and values to the current row
            dictCurrentRow.update(valueColumns)

        # Return the row
            return dictCurrentRow

        except:
            print("ERROR:\n")


        return None

    @staticmethod
    def getSignalTags(userId):

        try:
            # body = {"signalName": signalName,
            #         "userId": userId
            #         }

            payload = {'userId': userId}

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                         path='/sdk/get-tag', params=payload)
            jsonResult = post_req.get();

            if len(jsonResult) == 1:
                if jsonResult[0].get('messageCode') == '0045':
                    return jsonResult.get('message')

            return jsonResult


        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None

    @staticmethod
    def getAlarmTags(userId):

        try:
            # body = {"signalName": signalName,
            #         "userId": userId
            #         }

            payload = {'userId': userId}

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/alarm/get-tag-alarm', params=payload)
            jsonResult = post_req.get();

            if len(jsonResult) == 1 and jsonResult[0].__contains__('messageCode'):
                if jsonResult[0].get('messageCode') == '0045':
                    return jsonResult['message']

            return jsonResult


        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None


    @staticmethod
    def _convertJsonToDataFrame(jsonResponse, signalNames):
        """
        Converts given json_response to pandas DataFrame
        :param jsonResponse: Query result in json_response format
        :param signal_names: List of signal names in query to set as columns name of DataFrame
        :return: DataFrame object of json_response
        """
        try:
        # Convert JSON to LIST
        #     listResult = json.loads(jsonResponse)
        #     print(listResult)

        # list_result->dictionary->DataFram

            mylist = map(lambda x: Mind._mapHistorian2DataFrame(x, signalNames), jsonResponse)
            list_1 = list(mylist)
            dataframe= pd.DataFrame(list_1)
            return dataframe

        except:
            print("ERROR:\n")
        return None

    @staticmethod
    def _convertJsonToListOfDict(listResult,signalNames,signalIds) :
        ii = 0
        totaldict = {}
        for i in listResult:
            dict = i
            mediumlist = dict.get(str(signalIds[ii]))
            listofvaluetimetotal = []
            for l in mediumlist:

                time = l.get('TIME')
                value = l.get('VALUE')
                listofvaluetime = [time, value]
                listofvaluetimetotal.append(listofvaluetime)
            totaldict[signalNames[ii]] = listofvaluetimetotal
            ii = ii + 1

        print(totaldict)
        return totaldict

    @staticmethod
    def _validate(date_text):
        try:
            datetime.datetime.strptime(date_text, '%Y/%m/%d-%H:%M:%S')
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY/MM/DD-HH:mm:SS")

    @staticmethod
    def createAlarm(alarmName , alarmDescription,userId):

        try:
            body = {"alarmName": alarmName,
                    "description": alarmDescription,

                    "userId": userId
                    }

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/alarm/create-alarm', params=body)
            jsonResult = post_req.post();

            if jsonResult.get('messageCode') == '0000':
                return True
            else:
                return False

        # except urllib.error.HTTPError as err:
        #     print("{}\nError Code:{}, URL:{}".format(err, err.code, err.filename))

        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None

    @staticmethod
    def removeAlarm(alarmName, userId):

        try:
            body = {"alarmName": alarmName,
                    "userId": userId
                    }

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/alarm/remove-alarm', params=body)

            jsonResult = post_req.post();

            if jsonResult['messageCode'] == '0000':
                return True
            else:
                return False

            return jsonResult


        except KeyError as err:
            print("ERROR: Signal Name {} not found!\n".format(err))
        return None

    @staticmethod
    def setAlarm(alarm_name,alarm_comment,active_status,user_id):

        if isinstance(alarm_comment, str) and isinstance(active_status, bool) \
                and isinstance(user_id, str) and isinstance(alarm_name, str):
            print("this is ok")
            body = {"alarmName": alarm_name,
                    "alarmComment": alarm_comment,
                    "activeStatus": active_status,
                    "userId": user_id

                    }

            post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/alarm/set-alarm', params=body)
            json_result = post_req.post()
            return json_result
        else:

            return 'Input Type Is Not Correct. It Must Be (str,str,bool,str)'

    @staticmethod
    def getLastAlarm():

        post_req = Rest(f'http://{Constants.GATEWAY_SERVER_IP}:{Constants.GATEWAY_PORT}',
                            path='/alarm/get-last-alarm')
        json_result = post_req.get()

        return json_result



